/* 
    saa7185 - Philips SAA7185B video encoder driver

    Copyright (C) 1998 Dave Perks <dperks@ibm.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <linux/signal.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <linux/types.h>
#include <linux/wrapper.h>

#include <linux/videodev.h>
#include <linux/version.h>
#include <asm/uaccess.h>

#include "/usr/src/linux/drivers/char/i2c.h"
#include "buz.h"
#include "video_encoder.h"

#define DEBUG(x)	x	/* Debug driver */	

/* ----------------------------------------------------------------------- */

struct saa7185
{
	struct i2c_bus	*bus;
	int		addr;
	unsigned char	reg[128];

	int		norm;
	int		mux;
	int		bright;
	int		contrast;
	int		hue;
	int		sat;
};

#define	I2C_SAA7185        0x88

#define I2C_DELAY   10

/* ----------------------------------------------------------------------- */

static int saa7185_write(struct saa7185 *dev, unsigned char subaddr, unsigned char data)
{
	int ack;
	unsigned long flags;

	LOCK_I2C_BUS(dev->bus);

	i2c_start(dev->bus);
	i2c_sendbyte(dev->bus, dev->addr, I2C_DELAY);
	i2c_sendbyte(dev->bus, subaddr, I2C_DELAY);
	ack = i2c_sendbyte(dev->bus, data, I2C_DELAY);
	dev->reg[subaddr] = data;
	i2c_stop(dev->bus);
	UNLOCK_I2C_BUS(dev->bus);
	return ack;
}

static int saa7185_write_block(struct saa7185 *dev, unsigned const char *data, unsigned int len)
{
	int ack;
	unsigned subaddr;
	unsigned long flags;

	while (len > 1) {
		LOCK_I2C_BUS(dev->bus);
		i2c_start(dev->bus);
		i2c_sendbyte(dev->bus, dev->addr, I2C_DELAY);
		ack = i2c_sendbyte(dev->bus, (subaddr = *data++), I2C_DELAY);
		ack = i2c_sendbyte(dev->bus, (dev->reg[subaddr] = *data++), I2C_DELAY);
		len -= 2;
		while (len > 1 && *data == ++subaddr) {
			data++;
			ack = i2c_sendbyte(dev->bus, (dev->reg[subaddr] = *data++), I2C_DELAY);
			len -= 2;
		}
		i2c_stop(dev->bus);
		UNLOCK_I2C_BUS(dev->bus);
	}
	return ack;
}

/* ----------------------------------------------------------------------- */

static int saa7185_attach(struct i2c_device *device)
{
	int i;
	struct saa7185 * encoder;

	static const unsigned char init[] = {

		0x3a, 0x0f,	/* CBENB=0, V656=0, VY2C=1, YUV2C=1, MY2C=1, MUV2C=1 */

		0x42, 0x6b,	/* OVLY0=107 */
		0x43, 0x00,	/* OVLU0=0	white */
		0x44, 0x00,	/* OVLV0=0 */
		0x45, 0x22,	/* OVLY1=34 */
		0x46, 0xac,	/* OVLU1=172	yellow */
		0x47, 0x0e,	/* OVLV1=14 */
		0x48, 0x03,	/* OVLY2=3 */
		0x49, 0x1d,	/* OVLU2=29	cyan */
		0x4a, 0xac,	/* OVLV2=172 */
		0x4b, 0xf0,	/* OVLY3=240 */
		0x4c, 0xc8,	/* OVLU3=200	green */
		0x4d, 0xb9,	/* OVLV3=185 */
		0x4e, 0xd4,	/* OVLY4=212 */
		0x4f, 0x38,	/* OVLU4=56	magenta */
		0x50, 0x47,	/* OVLV4=71 */
		0x51, 0xc1,	/* OVLY5=193 */
		0x52, 0xe3,	/* OVLU5=227	red */
		0x53, 0x54,	/* OVLV5=84 */
		0x54, 0xa3,	/* OVLY6=163 */
		0x55, 0x54,	/* OVLU6=84	blue */
		0x56, 0xf2,	/* OVLV6=242 */
		0x57, 0x90,	/* OVLY7=144 */
		0x58, 0x00,	/* OVLU7=0	black */
		0x59, 0x00,	/* OVLV7=0 */

		0x5a, 0x00,	/* CHPS=0 */
		0x5b, 0x76,	/* GAINU=118 */
		0x5c, 0xa5,	/* GAINV=165 */
		0x5d, 0x3c,	/* BLCKL=60 */
		0x5e, 0x3a,	/* BLNNL=58 */
		0x5f, 0x00,	/* NULL */
		0x60, 0xba,	/* CCRS=2, BLNVB=63 */
		0x61, 0x15,	/* FISE=1, PAL=0, SCBW=1, RTCE=0, YGS=1, INPI=0, DOWN=0 */
		0x62, 0xe6,	/* DECTYP=1, BSTA=102 */
		0x63, 0x1f,	/* FSC0 */
		0x64, 0x7c,	/* FSC1 */
		0x65, 0xf0,	/* FSC2 */
		0x66, 0x21,	/* FSC3 */

		0x6b, 0x91,	/* MODIN=2, PCREF=0, SCCLN=17 */
		0x6c, 0x00,	/* SRCV1=0, TRCV2=0, ORCV1=0, PRCV1=0, CBLF=0, ORCV2=0, PRCV2=0 */
		0x6d, 0x04,	/* SRCM1=1, CCEN=0 */
		0x6e, 0x00,	/* HTRIG=0 */
		0x6f, 0x00,	/* HTRIG=0 */
		0x70, 0x20,	/* PHRES=0, SBLN=1, VTRIG=0 */
		0x71, 0xf9,	/* BMRQ=0x0f9 */
		0x72, 0x83,	/* EMRQ=0x683 */
		0x73, 0x60,	/* EMRQ=0x683, BMRQ=0x0f9 */
		0x74, 0x00,	/* NULL */
		0x75, 0x00,	/* NULL */
		0x76, 0x00,	/* NULL */
		0x77, 0xf9,	/* BRCV=0x0f9 */
		0x78, 0x83,	/* ERCV=0x683 */
		0x79, 0x60,	/* ERCV=0x683, BRCV=0x0f9 */
		0x7a, 0x00,	/* FLC=0 */
		0x7b, 0x16,	/* FAL=22 */
		0x7c, 0xf4,	/* LAL=244 */
		0x7d, 0x00,	/* LAL=244, FAL=22 */
	};

	device->data = encoder = kmalloc(sizeof(struct saa7185), GFP_KERNEL);
	if (encoder == NULL)
		return -ENOMEM;

	MOD_INC_USE_COUNT;

	memset(encoder, 0, sizeof(struct saa7185));
	strcpy(device->name, "saa7185");
	encoder->bus = device->bus;
	encoder->addr = device->addr;

	i = saa7185_write_block(encoder, init, sizeof(init));
	if (i < 0)
	{
		printk("%s_attach: init status %d\n", device->name, i);
	} else {
		printk("%s_attach: chip status %x\n", device->name, i2c_read(encoder->bus, encoder->addr | 1));
	}
	return 0;
}


static int saa7185_detach(struct i2c_device *device)
{
	kfree(device->data);
	MOD_DEC_USE_COUNT;
	return 0;
}

static int saa7185_command(struct i2c_device *device, unsigned int cmd, void * arg)
{
	struct saa7185 * encoder = device->data;
	int * iarg = (int *)arg;

	switch (cmd)
	{
#if 0

	case DECODER_SET_NORM:
		encoder->norm = *iarg;
		break;

	case DECODER_SET_INPUT:	/* sets input mode 0-7 (modes 3 and 6 are Buz? */
		encoder->mux = *iarg & 0x07;
		/* select mode */
		saa7185_write(encoder, 0x02, (encoder->reg[0x02] & 0xf8) | encoder->mux);
		/* bypass chrominance trap for modes 4..7 */
		saa7185_write(encoder, 0x09, (encoder->reg[0x09] & 0x7f) | (encoder->mux > 3)? 0x80: 0);
		break;

	case DECODER_SET_BRIGHTNESS:
		/* We want 0 to 255 we get 0-65535 */
		encoder->bright = *iarg;
		saa7185_write(encoder, 0x0a, encoder->bright >> 8);
		break;

	case DECODER_SET_CONTRAST:
		/* We want 0 to 127 we get 0-65535 */
		encoder->contrast = *iarg;
		saa7185_write(encoder, 0x0b, encoder->contrast >> 9);
		break;

	case DECODER_SET_SATURATION:
		/* We want 0 to 127 we get 0-65535 */
		encoder->sat = *iarg;
		saa7185_write(encoder, 0x0c, encoder->sat >> 9);
		break;

	case DECODER_SET_HUE:
		/* We want -128 to 127 we get 0-65535 */
		encoder->hue = *iarg;
		saa7185_write(encoder, 0x0d, encoder->hue >> 8);
		break;
#endif

	default:
		return -EINVAL;
	}

	return 0;
}

/* ----------------------------------------------------------------------- */

struct i2c_driver i2c_driver_saa7185 =
{
	"saa7185",		/* name */
	I2C_DRIVERID_ENCODER,	/* ID */
	I2C_SAA7185, I2C_SAA7185+1,

	saa7185_attach,
	saa7185_detach,
	saa7185_command
};

EXPORT_NO_SYMBOLS;

#ifdef MODULE
int init_module(void)
#else
int saa7185_init(void)
#endif
{
	return i2c_register_driver(&i2c_driver_saa7185);
}



#ifdef MODULE

void cleanup_module(void)
{
	i2c_unregister_driver(&i2c_driver_saa7185);
}

#endif

/*
 * Local variables:
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * c-continued-statement-offset: 8
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
